#version 330
 
in vec3 intensity;
in vec2 texCoord;
in vec4 ShadowCoord;
in vec4 position;
in vec4 prevPosition;

uniform sampler2D colorTexture;
uniform sampler2DShadow shadowMap;
 
layout (location = 0) out vec4 fragColor;
layout (location = 1) out vec2 velocity;

#define EPSILON 0.005

vec2 calcVelocity() {

    vec2 a = (position.xy / position.w) ;
    vec2 b = (prevPosition.xy / prevPosition.w) ;
	return a - b;

}
 
//Samples the Shadowmap with a 5x5 Kernel
float PCF(vec3 coord) {
    float xOffset = 1.0/1920.0;
    float yOffset = 1.0/1080.0;
 
    float Factor = 0.0;
 
    for (int y = -2 ; y <= 2 ; y++) {
        for (int x = -2 ; x <= 2 ; x++) {
            vec2 Offsets = vec2(x * xOffset, y * yOffset);
            vec3 UVC = vec3(coord.xy + Offsets, coord.z - EPSILON);
           
		    Factor += texture(shadowMap, UVC);
        }
    }
        return (Factor/25.0);
}
 
void main() {
    vec4 color = texture(colorTexture, texCoord);
        
	vec3 biasCoord = vec3(ShadowCoord.xy, ShadowCoord.z - 0.005);
	
	float visibility = PCF(ShadowCoord.xyz);
	visibility = clamp (visibility, 0.5, 1);

	fragColor = vec4(color.rgb * intensity * visibility, color.a);
	velocity = calcVelocity();
}