#version 330

uniform sampler2D velocityTexture;
uniform sampler2D blurTexture;
uniform float blurScale;

out vec4 fragColor;

#define MAX_SAMPLES 16


void main() {
    vec2 texelSize = 1.0 / vec2(textureSize(blurTexture, 0));
    vec2 screenTexCoords = gl_FragCoord.xy * texelSize;
   
    // Calculate the correct velocity of the pixel
	vec2 velocity = texture(velocityTexture, screenTexCoords).rg;
    velocity *= blurScale;
   
     
    // Calculate the number of samples
    float speed = length(velocity / texelSize);  
    int nSamples = clamp(int(speed), 1, MAX_SAMPLES);
   
    fragColor = texture(blurTexture, screenTexCoords);
   
    for(int i = 1; i < nSamples; i++) {
        vec2 offset = velocity * (float(i) / float(nSamples) - 0.5);
        fragColor+= texture(blurTexture, screenTexCoords + offset);
    }

        fragColor /= float(nSamples);
}
